'''

 Filter to add to the INPUT DRAWABLE reflections on selected areas (one or more)
 to simulate the existence of ponds
 .................................................................................
 
 Release 1.0   initial
         1.0.1 commented out info messages
               added the possibility for the user to specify the width of the blur on margin
 
 Flow implemented to get the final result:       
 
 1. check whether there is a selection (mandatory)
 2. duplicate the source drawable ("ToReflectLayer")
 3. remove tha part below the "horizon" (defined as the top most pixel of the selection)
 4. resize the image to the double of the upper part
 5. duplicate the "ToReflectLayer" to have the "ReflectedLayer",
    flip it along the "horizon"
 6. check whether the user chose to compress the reflection and apply compression factor   
 7. duplicate the "ReflectedLayer" to have the "WaterLayer",
    offset to the horizon line,
    fill it with white, apply rgb noise, apply a gaussian blur, emboss it
 8. apply perspective to the "WaterLayer", apply a gaussian blur,
    set FG to a dark greenish-blue and apply a FG-to-transp gradient  
    set layer to Hardlight at 90%
 9. apply a gaussian blur to "ReflectedLayer", 
    warp it using the "WaterLayer"
 10.duplicate "WaterLayer" to get "PatternLayer"
    fill it using chosen pattern   
 11.merge all created layers and resize to image size
 12.restore original selection and cut whatever around
 13.add a new layer for distortions,
    fill with the pattern, emboss, apply whirl-pinch
 14.merge with the PatternLayer
 15.apply a little blur to the margin
 16.flatten if requested
 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
  
'''
#!/usr/bin/env python
from gimpfu import *
def ReflectOnPonds(inImage, inDrawable, inPattern, ScaleR, BBlur, doFlatten) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    pdb.gimp_context_set_antialias(True)
    pdb.gimp_context_set_feather(False)
    pdb.gimp_context_set_feather_radius(0, 0)
    pdb.gimp_context_set_sample_merged(False) 
    pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_COMPOSITE) 
    pdb.gimp_context_set_sample_threshold(0.00)
    pdb.gimp_context_set_sample_transparent(False)       # beware, if TRUE may change the area selected!
    pdb.gimp_context_set_interpolation(INTERPOLATION_LANCZOS)

    width = inImage.width
    height = inImage.height
    type = inImage.base_type
    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(inImage)

    # STEP1 : check whether there is a selection (mandatory)
    # ------------------------------------------------------
    (select, selx1, sely1, selx2, sely2) = pdb.gimp_drawable_mask_bounds (inDrawable)
    #gimp.message ("selection="+str(selx1)+","+str(sely1)+","+str(selx2)+","+str(sely2))    # 1.0.1
    #gimp.message ("select = "+str(select))
    pdb.gimp_layer_add_alpha (inDrawable)
    if select == 1 :    # there is a selection
        select_channel = pdb.gimp_selection_save (inImage)
        backup_layer = inDrawable.copy()
        selwidth = selx2 - selx1
        selheight = sely2 - sely1
    else :
        gimp.message ("Please make a selection")
        pdb.gimp_image_undo_group_end(inImage)
        pdb.gimp_displays_flush
        return(inDrawable)
    
    
    # STEP2 : duplicate the source drawable ("ToReflectLayer")
    # -------------------------------------------------------- 
    ToReflectLayer = inDrawable.copy()
    inImage.add_layer (ToReflectLayer, -1)
    ToReflectLayer.mode = NORMAL_MODE
    ToReflectLayer.name = "ToReflect Layer"
    ToReflectLayer.opacity = 100.0
    pdb.gimp_layer_add_alpha (ToReflectLayer)
    
    
    # STEP3 : remove tha part below the "horizon" (defined as the top most pixel of the selection)
    # ---------------------------------------------------------------------------------------------
    horizon = sely1
    #gimp.message ("horizon at pixel "+str(sely1))      # 1.0.1
    pdb.gimp_image_select_rectangle (inImage, CHANNEL_OP_REPLACE, 0, horizon, width, height)
    pdb.gimp_edit_clear (ToReflectLayer)
    
    
    # STEP4 : resize the image to the double of the upper part
    # --------------------------------------------------------
    pdb.gimp_image_resize (inImage, width, int(horizon*2), 0, 0)
    
    
    # STEP5 : duplicate the "ToReflectLayer" to have the "ReflectedLayer",
    #         flip it along the "horizon" and resize "ToReflectLayer" to image size
    # -----------------------------------------------------------------------------
    ReflectedLayer = ToReflectLayer.copy()
    inImage.add_layer (ReflectedLayer, -1)
    ReflectedLayer.mode =  NORMAL_MODE
    ReflectedLayer.name = "Reflected Layer"
    ReflectedLayer.opacity = 100.0 
    pdb.gimp_layer_add_alpha (ReflectedLayer)
    
    pdb.gimp_selection_none (inImage)
    ReflectedLayer = pdb.gimp_item_transform_flip_simple (ReflectedLayer,
                                                          ORIENTATION_VERTICAL, 0, horizon)
    
    pdb.gimp_layer_resize_to_image_size (ToReflectLayer)
    W = ReflectedLayer.width
    H = ReflectedLayer.height
    # gimp.message ("W,H="+str(W)+","+str(H))       # 1.0.1
    
    # try compress here
    if ScaleR < 1.0 :
        compress = H * (1.0-ScaleR)  # compress the selection height
        #gimp.message ("compress = "+str(compress))     # 1.0.1
        newy1 = horizon*2 - height + compress/4
        newy2 = newy1 + H - compress
        # gimp.message (str(0)+","+str(newy1)+","+str(W)+","+str(newy2))        # 1.0.1
        FT=pdb.gimp_item_transform_scale (ReflectedLayer, 0, newy1, W, newy2)
        #pdb.gimp_floating_sel_anchor (FT)
    # end try
    
    #pdb.gimp_displays_flush
    #return
    
    # STEP6 : duplicate the "ReflectedLayer" to have the "WaterLayer",
    #         offset to the horizon line and resize to horizon,
    #         fill it with white, apply rgb noise, apply a gaussian blur, emboss it
    # -----------------------------------------------------------------------------
    WaterLayer = ReflectedLayer.copy()
    inImage.add_layer (WaterLayer, -1)
    pdb.gimp_layer_add_alpha (WaterLayer)
    WaterLayer.name = "Water Layer"
    WaterLayer.opacity = 100.0
    pdb.gimp_layer_set_offsets (WaterLayer, 0, horizon)
    pdb.gimp_image_select_rectangle (inImage, CHANNEL_OP_REPLACE, 0, horizon, width, H)
    pdb.gimp_edit_clear (WaterLayer)
    ###pdb.gimp_layer_resize  (WaterLayer, width, horizon, 0, 0)

    pdb.gimp_drawable_fill (WaterLayer, WHITE_FILL)
    pdb.plug_in_rgb_noise (inImage, WaterLayer, 0, 1, 1.0, 1.0, 1.0, 0)
    pdb.plug_in_gauss_iir2 (inImage, WaterLayer, 20, 5)
    pdb.plug_in_emboss (inImage, WaterLayer, 90, 50.0, 10, 1)
    
    (select, wx1, wy1, wx2, wy2) = pdb.gimp_drawable_mask_bounds (WaterLayer)
    #gimp.message ("water layer area ="+str(wx1)+","+str(wy1)+","+str(wx2)+","+str(wy2))        # 1.0.1
    IH = pdb.gimp_image_height (inImage)
    #gimp.message ("image height = "+str(IH))       # 1.0.1
    #pdb.gimp_displays_flush
    #return
    
    # STEP7 : apply perspective to the "WaterLayer", apply a gaussian blur,
    #         set FG to a dark greenish-blue and apply a FG-to-transp gradient
    #         set layer to Multiply at 90%
    # ------------------------------------------------------------------------
    inWave = 2.0
    perspective_width = W * inWave 
    fs=pdb.gimp_item_transform_perspective (WaterLayer,
                                            0, horizon,  # upper left
                                            W, horizon,  # upper right
                                            0-perspective_width,  horizon*2,  # lower left
                                            W+perspective_width,  horizon*2  # lower right
                                           )
    pdb.gimp_floating_sel_anchor (fs)
    
    pdb.plug_in_gauss_iir2 (inImage, WaterLayer, 20, 2)
    pdb.gimp_context_set_foreground ((32, 64, 56, 255))
    pdb.gimp_edit_blend (WaterLayer, FG_TRANSPARENT_MODE, NORMAL_MODE, GRADIENT_LINEAR,
                         90.0, 0, 0, 0, 0, 0, 0, 0, W/2, 0, W/2, H)

    WaterLayer.mode = HARDLIGHT_MODE    
    WaterLayer.opacity = 90.0            
    
    #pdb.gimp_displays_flush
    #return
    
    # STEP8 : apply a gaussian blur to "ReflectedLayer", and
    #         warp it using the "WaterLayer"
    # ------------------------------------------------------ 
    pdb.plug_in_gauss_iir2 (inImage, ReflectedLayer, 1, 1)
    
    inWarp = 30.0
    pdb.plug_in_warp (inImage,              #image
                      ReflectedLayer,       #drawable
                      float(inWarp),        #amount
                      WaterLayer,           #warp-map
                      1,                    #iterations
                      0.0,                  #dither
                      0.0,                  #angle
                      1,                    #wrap-type = SMEAR
                      WaterLayer,           #drawable for magnitude
                      0,                    #use it = no
                      1,                    #substeps
                      0,                    #grad map
                      0.0,                  #grad scale
                      0,                    #vectore map
                      0.0,                  #vector scale
                      0.0)                  #vector angle
    
    #pdb.gimp_displays_flush
    #return
    
    # STEP9 : duplicate "WaterLayer" to get "PatternLayer"
    #         fill it using chosen pattern
    # ------------------------------------------------------------------------ 
    PatternLayer = pdb.gimp_layer_new_from_drawable (WaterLayer, inImage) 
    pdb.gimp_image_add_layer (inImage, PatternLayer, -1)
    PatternLayer.name = "PatternLayer"
    PatternLayer.mode = MULTIPLY_MODE
    PatternLayer_opacity = 80.0
    pdb.gimp_context_set_pattern(inPattern)
    pdb.gimp_edit_fill (PatternLayer, PATTERN_FILL)
    
    # merge all created layers
    Merged1 = pdb.gimp_image_merge_down (inImage, PatternLayer, 2)   # pattern merged over water
    Merged1.mode = HARDLIGHT_MODE   # restore waterlayermode
    Merged1.opacity = 90.0          # restare waterlayer opacity
    Merged2 = pdb.gimp_image_merge_down (inImage, Merged1, 2)   # ..merged over reflected (normal)
    Merged3 = pdb.gimp_image_merge_down (inImage, Merged2, 2)   # ..merged over toreflect (normal)
    Merged3.name = "Drawable with full reflection"
 
    # try to cut everything outside the selection
    pdb.gimp_selection_load (select_channel)
    pdb.gimp_selection_invert (inImage)
    pdb.gimp_edit_clear (Merged3)
    pdb.gimp_selection_invert (inImage)
    # end try
    
    #pdb.gimp_displays_flush
    #return
    
    # STEP10: restore original image size
    # ------------------------------------------------------------------------
 
    ReflectOnPondLayer = Merged3
    ReflectOnPondLayer.name = "Drawable with full reflection only on the selection"
    pdb.gimp_image_resize (inImage, width, height, 0, 0)
 
    
    # STEP11: add on top a new layer
    #         fill it with waves of chaos
    # ------------------------------------ 
    ChaosLayer = pdb.gimp_layer_new (inImage, width, height, RGBA_IMAGE, "Pattern", 25.0, HARDLIGHT_MODE) 
    pdb.gimp_image_add_layer (inImage, ChaosLayer, -1)
    pdb.gimp_edit_fill (ChaosLayer, PATTERN_FILL)
    ChaosLayer.name = "Distortion Layer"
    #pdb.plug_in_waves_of_chaos (inImage, ChaosLayer, 12, 1, 12, 5, 8, 6)
    pdb.plug_in_emboss (inImage, ChaosLayer, 30, 25, 15, 0)
    pdb.plug_in_whirl_pinch (inImage, ChaosLayer, 75, 0.75, 1.5)
    #pdb.plug_in_wind (inImage, ChaosLayer, 10, 1, 35, 0, 0)
 
    #pdb.gimp_displays_flush
    #return
    
    # try
    MP=pdb.gimp_image_merge_down (inImage, ChaosLayer, CLIP_TO_IMAGE)
    # tentative blur on margin
    pdb.gimp_context_set_sample_transparent(True)       # beware, if TRUE may change the area selected!
    pdb.gimp_image_select_color (inImage, CHANNEL_OP_REPLACE, MP, ((0,0,0,0)))
    pdb.gimp_selection_invert (inImage)
    pdb.gimp_selection_shrink (inImage, BBlur)
    pdb.gimp_selection_invert (inImage)     # 1.0.1
    pdb.plug_in_gauss_iir2 (inImage, MP, BBlur/2, BBlur/2)      # was 3 - 1.0.1
    pdb.gimp_selection_invert (inImage)
    
    if doFlatten == True :
        flattenedLayer = pdb.gimp_image_merge_down (inImage, MP, 0)
        flattenedLayer.name = name+", with Reflections on Ponds!"   # 1.0.1
        pdb.gimp_selection_none (inImage)
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_image_undo_group_end(inImage)
    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    
    if doFlatten == True :
        return(flattenedLayer)
    else :
        return(ReflectOnPondLayer)

# This is the plugin registration function
register(
    "ReflectOnPonds",    
    "to create a new image with partial reflection on a selection (required) of input image.",   
    "This script creates new image with a partial reflection on a selection (required) of the input drawable (a 'pond').",
    "Diego", 
    "Diego Nassetti ", 
    "September 2016",
    "ReflectOnPonds (v.1.0.1)", 
    "RGB*,GRAY*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_PATTERN, "pattern", "Pattern to overlay", "waterblu.png"),
      (PF_ADJUSTMENT, "scale", "Reflection scale", 0.90, (0.66, 1.00, 0.05)),
      (PF_ADJUSTMENT, "bblur", "Border blur", 6, (1, 30, 1)),
      (PF_TOGGLE, "flatten", "Flatten image?", False),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    ReflectOnPonds,
    menu="<Image>/Diego/Artistic",
    )

main()
